@@ -1,7 +1,7 @@
 # This file is part of Hypothesis, which may be found at
 # https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2020 David R. MacIver
+# Most of this work is copyright (C) 2013-2021 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -293,8 +293,7 @@ def _category_key(exclude, include):
     assert include.issubset(cs)
     assert exclude.issubset(cs)
     include -= exclude
-    result = tuple(c for c in cs if c in include)
-    return result
+    return tuple(c for c in cs if c in include)
 
 
 def _query_for_key(key):
